@@ -25,6 +25,7 @@
 #include <resources/StandardBackendUpdater.h>
 
 #include <QDebug>
+#include <QDesktopServices>
 #include <QFileSystemWatcher>
 #include <QFutureWatcher>
 #include <QHash>
@@ -111,7 +112,7 @@ static void setWhenAvailable(const QDBusPendingReply<T> &pending, W func, QObjec
 
 QString PackageKitBackend::locateService(const QString &filename)
 {
-    return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("applications/") + filename);
+    return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1StringView("applications/") + filename);
 }
 
 Delay::Delay()
@@ -132,7 +133,18 @@ PackageKitBackend::PackageKitBackend(QObject *parent)
     , m_refresher(nullptr)
     , m_isFetching(0)
     , m_reviews(OdrsReviewsBackend::global())
+    , m_reportToDistroAction(
+          new DiscoverAction(u"tools-report-bug-symbolic"_s,
+                             i18nc("@action:button %1 is the distro name", "Report This Issue to %1", AppStreamIntegration::global()->osRelease()->name()),
+                             this))
 {
+    connect(m_reportToDistroAction, &DiscoverAction::triggered, this, [] {
+        const auto url = QUrl(AppStreamIntegration::global()->osRelease()->bugReportUrl());
+        if (!QDesktopServices::openUrl(url)) {
+            qCWarning(LIBDISCOVER_BACKEND_PACKAGEKIT_LOG) << "Failed to open bug report url" << url;
+        }
+    });
+
     QTimer *t = new QTimer(this);
     connect(t, &QTimer::timeout, this, &PackageKitBackend::checkForUpdates);
     t->setInterval(60 * 60 * 1000);
@@ -301,15 +313,6 @@ void PackageKitBackend::acquireFetching(bool f)
     Q_ASSERT(m_isFetching >= 0);
 }
 
-static bool loadAppStream(AppStream::Pool *appdata)
-{
-    bool correct = appdata->load();
-    if (!correct) {
-        qWarning() << "PackageKitBackend: Could not open the AppStream metadata pool" << appdata->lastError();
-    }
-    return correct;
-}
-
 void PackageKitBackend::reloadPackageList()
 {
     acquireFetching(true);
@@ -363,20 +366,21 @@ void PackageKitBackend::reloadPackageList()
         }
     };
 
-    // We do not want to block the GUI for very long, so we load appstream pools via queued
-    // invocation both for the actual load and then for the post-load state changes.
-    QMetaObject::invokeMethod(
-        this,
-        [this, loadDone] {
-            auto result = loadAppStream(m_appdata.get());
-            QMetaObject::invokeMethod(
-                this,
-                [loadDone, result] {
-                    loadDone(result);
-                },
-                Qt::QueuedConnection);
-        },
-        Qt::QueuedConnection);
+    connect(m_appdata.get(), &AppStream::Pool::loadFinished, this, [this, loadDone](bool success) {
+        m_appdataLoaded = true;
+        if (!success) {
+            qWarning() << "PackageKitBackend: Could not open the AppStream metadata pool" << m_appdata->lastError();
+        }
+        // We do not want to block the GUI for very long, so we load appstream pools via queued
+        // invocation for the post-load state changes.
+        QMetaObject::invokeMethod(
+            this,
+            [loadDone, success] {
+                loadDone(success);
+            },
+            Qt::QueuedConnection);
+    });
+    m_appdata->loadAsync();
 }
 
 AppPackageKitResource *PackageKitBackend::addComponent(const AppStream::Component &component) const
@@ -760,7 +764,8 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
 
             auto resources = kFilter<QVector<AbstractResource *>>(m_packages.packages, [](AbstractResource *resource) {
                 auto pkResource = qobject_cast<PackageKitResource *>(resource);
-                return resource->type() != AbstractResource::Technical && pkResource && !pkResource->isCritical() && !pkResource->extendsItself();
+                // Neither PackageKitResource or its subclass AppPackageKitResource can have type == ApplicationSupport
+                return resource->type() != AbstractResource::System && pkResource && !pkResource->isCritical() && !pkResource->extendsItself();
             });
             stream->sendResources(kTransform<QVector<StreamResult>>(resources, [](auto resource) {
                 return StreamResult(resource, 0);
@@ -768,31 +773,35 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
         });
     } else {
         return deferredResultStream(u"PackageKitStream-search"_s, [this, filter = filter](PKResultsStream *stream) {
-            AppStream::ComponentBox components(AppStream::ComponentBox::FlagNone);
-            if (!filter.search.isEmpty()) {
-                components = m_appdata->search(filter.search);
-            } else if (filter.category) {
-                components = AppStreamUtils::componentsByCategories(m_appdata.get(), filter.category, AppStream::Bundle::KindUnknown);
-            } else {
-                components = m_appdata->components();
-            }
-
-            QSet<QString> ids;
-            kFilterInPlace<AppStream::ComponentBox>(components, [&ids](const AppStream::Component &component) {
-                if (ids.contains(component.id())) {
-                    return false;
+            auto loadComponents = [](const auto &filter, const auto &appdata) -> QCoro::Task<AppStream::ComponentBox> {
+                AppStream::ComponentBox components(AppStream::ComponentBox::FlagNone);
+                if (!filter.search.isEmpty()) {
+                    components = appdata->search(filter.search);
+                } else if (filter.category) {
+                    components = co_await AppStreamUtils::componentsByCategoriesTask(appdata.get(), filter.category, AppStream::Bundle::KindUnknown);
+                } else {
+                    components = appdata->components();
                 }
-                ids.insert(component.id());
-                return true;
-            });
-            if (!ids.isEmpty()) {
-                const auto resources = kFilter<QVector<StreamResult>>(resultsByComponents(components), [](const StreamResult &res) {
-                    return !qobject_cast<PackageKitResource *>(res.resource)->extendsItself();
+                co_return components;
+            };
+            QCoro::connect(loadComponents(filter, m_appdata), this, [this, stream, filter](auto &&components) {
+                QSet<QString> ids;
+                kFilterInPlace<AppStream::ComponentBox>(components, [&ids](const AppStream::Component &component) {
+                    if (ids.contains(component.id())) {
+                        return false;
+                    }
+                    ids.insert(component.id());
+                    return true;
                 });
-                stream->sendResources(resources, filter.state != AbstractResource::Broken);
-            } else {
-                stream->finish();
-            }
+                if (!ids.isEmpty()) {
+                    const auto resources = kFilter<QVector<StreamResult>>(resultsByComponents(components), [](const StreamResult &res) {
+                        return !qobject_cast<PackageKitResource *>(res.resource)->extendsItself();
+                    });
+                    stream->sendResources(resources, filter.state != AbstractResource::Broken);
+                } else {
+                    stream->finish();
+                }
+            });
         });
     }
 }
@@ -988,9 +997,16 @@ void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
     }
 
     if (!m_updater->isDistroUpgrade() && !PackageKit::Daemon::global()->offline()->upgradeTriggered()) {
-        auto nextRelease = AppStreamIntegration::global()->getDistroUpgrade(m_appdata.get());
-        if (nextRelease) {
-            foundNewMajorVersion(*nextRelease);
+        const auto loadDone = [this] {
+            auto nextRelease = AppStreamIntegration::global()->getDistroUpgrade(m_appdata.get());
+            if (nextRelease) {
+                foundNewMajorVersion(*nextRelease);
+            }
+        };
+        if (m_appdataLoaded) {
+            loadDone();
+        } else {
+            connect(m_appdata.get(), &AppStream::Pool::loadFinished, this, loadDone);
         }
     }
 }
@@ -998,7 +1014,7 @@ void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
 void PackageKitBackend::foundNewMajorVersion(const AppStream::Release &release)
 {
     const QString upgradeVersion = release.version();
-    const QString newDistroVersionText = AppStreamIntegration::global()->osRelease()->name() + QStringLiteral(" ") + upgradeVersion;
+    const QString newDistroVersionText = AppStreamIntegration::global()->osRelease()->name() + QLatin1Char(' ') + upgradeVersion;
 
     QString info;
     // Message to display when:
@@ -1159,6 +1175,14 @@ InlineMessage *PackageKitBackend::explainDysfunction() const
     if (!error.isEmpty()) {
         return new InlineMessage(InlineMessage::Error, QStringLiteral("network-disconnect"), error);
     }
+
+    if (!PackageKit::Daemon::isRunning()) {
+        return new InlineMessage(InlineMessage::Error,
+                                 u"run-build-prune-symbolic"_s,
+                                 i18nc("@info", "The background service (PackageKit) stopped unexpectedly. It may have crashed."),
+                                 m_reportToDistroAction);
+    }
+
     return AbstractResourcesBackend::explainDysfunction();
 }
 
@@ -1177,5 +1201,19 @@ void PackageKitBackend::loadAllPackages()
     m_allPackagesLoaded = true;
 }
 
+void PackageKitBackend::aboutTo(AboutToAction action)
+{
+    PackageKit::Offline::Action packageKitAction;
+    switch (action) {
+    case Reboot:
+        packageKitAction = PackageKit::Offline::ActionReboot;
+        break;
+    case PowerOff:
+        packageKitAction = PackageKit::Offline::ActionPowerOff;
+        break;
+    }
+    m_updater->setOfflineUpdateAction(packageKitAction);
+}
+
 #include "PackageKitBackend.moc"
 #include "moc_PackageKitBackend.cpp"
